@@ -66,7 +66,7 @@ extern "C" {
 #define NC_FILL_BYTE    ((signed char)-127)
 #define NC_FILL_CHAR    ((char)0)
 #define NC_FILL_SHORT   ((short)-32767)
-#define NC_FILL_INT     (-2147483647L)
+#define NC_FILL_INT     (-2147483647)
 #define NC_FILL_FLOAT   (9.9692099683868690e+36f) /* near 15 * 2^119 */
 #define NC_FILL_DOUBLE  (9.9692099683868690e+36)
 #define NC_FILL_UBYTE   (255)
@@ -116,22 +116,24 @@ extern "C" {
 /* Define the ioflags bits for nc_create and nc_open.
    currently unused:
         0x0002
-        0x0040
-        0x0080
    and the whole upper 16 bits
 */
 
 #define NC_NOWRITE       0x0000 /**< Set read-only access for nc_open(). */
 #define NC_WRITE         0x0001 /**< Set read-write access for nc_open(). */
+
 #define NC_CLOBBER       0x0000 /**< Destroy existing file. Mode flag for nc_create(). */
 #define NC_NOCLOBBER     0x0004 /**< Don't destroy existing file. Mode flag for nc_create(). */
 
 #define NC_DISKLESS      0x0008  /**< Use diskless file. Mode flag for nc_open() or nc_create(). */
-#define NC_MMAP          0x0010  /**< Use diskless file with mmap. Mode flag for nc_open() or nc_create(). */
+#define NC_MMAP          0x0010  /**< \deprecated Use diskless file with mmap. Mode flag for nc_open() or nc_create()*/
 
 #define NC_64BIT_DATA    0x0020  /**< CDF-5 format: classic model but 64 bit dimensions and sizes */
 #define NC_CDF5          NC_64BIT_DATA  /**< Alias NC_CDF5 to NC_64BIT_DATA */
 
+#define NC_UDF0          0x0040  /**< User-defined format 0. */
+#define NC_UDF1          0x0080  /**< User-defined format 1. */
+
 #define NC_CLASSIC_MODEL 0x0100 /**< Enforce classic model on netCDF-4. Mode flag for nc_create(). */
 #define NC_64BIT_OFFSET  0x0200  /**< Use large (64-bit) file offsets. Mode flag for nc_create(). */
 
@@ -149,14 +151,17 @@ Use this in mode flags for both nc_create() and nc_open(). */
 
 /** Turn on MPI I/O.
 Use this in mode flags for both nc_create() and nc_open(). */
-#define NC_MPIIO         0x2000
+#define NC_MPIIO         0x2000 /**< \deprecated */
 /** Turn on MPI POSIX I/O.
 Use this in mode flags for both nc_create() and nc_open(). */
-#define NC_MPIPOSIX      0x4000 /**< \deprecated As of libhdf5 1.8.13. */
+#define NC_MPIPOSIX      NC_MPIIO /**< \deprecated As of libhdf5 1.8.13. Now an alias */
 
-#define NC_INMEMORY      0x8000  /**< Read from memory. Mode flag for nc_open() or nc_create(). */
+#define NC_PERSIST       0x4000  /**< Save diskless contents to disk. Mode flag for nc_open() or nc_create() */
+#define NC_INMEMORY      0x8000  /**< Read from memory. Mode flag for nc_open() or nc_create() */
 
-#define NC_PNETCDF       (NC_MPIIO) /**< Use parallel-netcdf library; alias for NC_MPIIO. */
+#define NC_PNETCDF       (NC_MPIIO) /**< \deprecated Use PnetCDF library; alias for NC_MPIIO. */
+
+#define NC_MAX_MAGIC_NUMBER_LEN 8 /**< Max len of user-defined format magic number. */
 
 /** Format specifier for nc_set_default_format() and returned
  *  by nc_inq_format. This returns the format as provided by
@@ -207,6 +212,8 @@ Use this in mode flags for both nc_create() and nc_open(). */
 #define NC_FORMATX_PNETCDF   (4)
 #define NC_FORMATX_DAP2      (5)
 #define NC_FORMATX_DAP4      (6)
+#define NC_FORMATX_UDF0      (8)
+#define NC_FORMATX_UDF1      (9)
 #define NC_FORMATX_UNDEFINED (0)
 
   /* To avoid breaking compatibility (such as in the python library),
@@ -255,7 +262,7 @@ created with the ::NC_CLASSIC_MODEL flag.
 
 As a rule, NC_MAX_VAR_DIMS <= NC_MAX_DIMS.
 
-NOTE: The NC_MAX_DIMS, NC_MAX_ATTRS, and NC_MAX_VARS limits 
+NOTE: The NC_MAX_DIMS, NC_MAX_ATTRS, and NC_MAX_VARS limits
       are *not* enforced after version 4.5.0
 */
 /**@{*/
@@ -418,6 +425,8 @@ by the desired type. */
 /* Misc. additional errors */
 #define NC_ENOTFOUND     (-90)      /**< No such file */
 #define NC_ECANTREMOVE   (-91)      /**< Can't remove file */
+#define NC_EINTERNAL     (-92)      /**< NetCDF Library Internal Error */
+#define NC_EPNETCDF      (-93)      /**< Error at PnetCDF layer */
 
 /* The following was added in support of netcdf-4. Make all netcdf-4
    error codes < -100 so that errors can be added to netcdf-3 if
@@ -458,7 +467,8 @@ by the desired type. */
 #define NC_EFILTER       (-132)    /**< Filter operation failed. */
 #define NC_ERCFILE       (-133)    /**< RC file failure */
 #define NC_ENULLPAD      (-134)    /**< Header Bytes not Null-Byte padded */
-#define NC4_LAST_ERROR   (-135)    /**< @internal All netCDF errors > this. */
+#define NC_EINMEMORY     (-135)    /**< In-memory file error */
+#define NC4_LAST_ERROR   (-136)    /**< @internal All netCDF errors > this. */
 
 /** @internal This is used in netCDF-4 files for dimensions without
  * coordinate vars. */
@@ -502,6 +512,14 @@ nc_inq_libvers(void);
 EXTERNL const char *
 nc_strerror(int ncerr);
 
+/* Set up user-defined format. */
+typedef struct NC_Dispatch NC_Dispatch;
+EXTERNL int
+nc_def_user_format(int mode_flag, NC_Dispatch *dispatch_table, char *magic_number);
+
+EXTERNL int
+nc_inq_user_format(int mode_flag, NC_Dispatch **dispatch_table, char *magic_number);
+
 EXTERNL int
 nc__create(const char *path, int cmode, size_t initialsz,
          size_t *chunksizehintp, int *ncidp);
@@ -879,8 +897,8 @@ nc_inq_var_filter(int ncid, int varid, unsigned int* idp, size_t* nparams, unsig
 EXTERNL int
 nc_set_fill(int ncid, int fillmode, int *old_modep);
 
-/* Set the default nc_create format to NC_FORMAT_CLASSIC,
- * NC_FORMAT_64BIT, NC_FORMAT_NETCDF4, etc */
+/* Set the default nc_create format to NC_FORMAT_CLASSIC, NC_FORMAT_64BIT,
+ * NC_FORMAT_CDF5, NC_FORMAT_NETCDF4, or NC_FORMAT_NETCDF4_CLASSIC */
 EXTERNL int
 nc_set_default_format(int format, int *old_formatp);
 
@@ -1724,8 +1742,6 @@ EXTERNL int
 nc_get_var_ubyte(int ncid, int varid, unsigned char *ip);
 /* End Deprecated */
 
-#ifdef LOGGING
-
 /* Set the log level. 0 shows only errors, 1 only major messages,
  * etc., to 5, which shows way too much information. */
 EXTERNL int
@@ -1735,12 +1751,6 @@ nc_set_log_level(int new_level);
    nc_log_level(NC_TURN_OFF_LOGGING) */
 #define NC_TURN_OFF_LOGGING (-1)
 
-#else /* not LOGGING */
-
-#define nc_set_log_level(e) /**< Get rid of these calls. */
-
-#endif /* LOGGING */
-
 /* Show the netCDF library's in-memory metadata for a file. */
 EXTERNL int
 nc_show_metadata(int ncid);
@@ -1789,11 +1799,6 @@ EXTERNL int
 nctypelen(nc_type datatype);
 
 /* Begin v2.4 backward compatibility */
-/*
- * defining NO_NETCDF_2 to the preprocessor
- * turns off backward compatibility declarations.
- */
-#ifndef NO_NETCDF_2
 
 /** Backward compatible alias. */
 /**@{*/
@@ -1829,7 +1834,7 @@ EXTERNL int ncerr;
 #define NC_VERBOSE      2  /**< For V2 API, be verbose on error. */
 
 /** V2 API error handling. Default is (NC_FATAL | NC_VERBOSE). */
-EXTERNL int ncopts;     
+EXTERNL int ncopts;
 
 EXTERNL void
 nc_advise(const char *cdf_routine_name, int err, const char *fmt,...);
@@ -1956,19 +1961,27 @@ ncrecget(int ncid, long recnum, void **datap);
 EXTERNL int
 ncrecput(int ncid, long recnum, void *const *datap);
 
-EXTERNL int
-nc_finalize();
+/* This function may be called to force the library to
+   initialize itself. It is not required, however.
+*/
+EXTERNL int nc_initialize(void);
 
-/* End v2.4 backward compatibility */
-#endif /*!NO_NETCDF_2*/
+/* This function may be called to force the library to
+   cleanup global memory so that memory checkers will not
+   report errors. It is not required, however.
+*/
+EXTERNL int nc_finalize(void);
 
 #if defined(__cplusplus)
 }
 #endif
 
 /* Define two hard-coded functionality-related
-   macros, but this is not going to be
-   standard practice. */
+   (as requested by community developers) macros.
+   This is not going to be standard practice.
+   Don't remove without an in-place replacement of some sort,
+   the are now (for better or worse) used by downstream
+   software external to Unidata. */
 #ifndef NC_HAVE_RENAME_GRP
 #define NC_HAVE_RENAME_GRP /*!< rename_grp() support. */
 #endif
@@ -1977,4 +1990,6 @@ nc_finalize();
 #define NC_HAVE_INQ_FORMAT_EXTENDED /*!< inq_format_extended() support. */
 #endif
 
+#define NC_HAVE_META_H
+
 #endif /* _NETCDF_ */
