@@ -8,12 +8,10 @@ Research/Unidata. See \ref copyright file for more info.
 
 #include "ncdispatch.h"
 
-#undef VARS_USES_VARM
-#ifndef VARS_USES_VARM
+/*!
+  \internal
 
-/*! \internal
-
- */
+*/
 struct GETodometer {
     int            rank;
     size_t         index[NC_MAX_VAR_DIMS];
@@ -24,13 +22,19 @@ struct GETodometer {
 };
 
 
-/** \internal
-
+/**
+ * @internal Initialize odometer.
+ *
+ * @param odom Pointer to odometer.
+ * @param rank
+ * @param start Start indices.
+ * @param edges Counts.
+ * @param stride Strides.
+ *
  */
 static void
-odom_init(struct GETodometer* odom,
-	    int rank,
-	    const size_t* start, const size_t* edges, const ptrdiff_t* stride)
+odom_init(struct GETodometer* odom, int rank, const size_t* start,
+          const size_t* edges, const ptrdiff_t* stride)
 {
     int i;
     memset(odom,0,sizeof(struct GETodometer));
@@ -45,8 +49,12 @@ odom_init(struct GETodometer* odom,
     }
 }
 
-/** \internal
-
+/**
+ * @internal Return true if there is more.
+ *
+ * @param odom Pointer to odometer.
+ *
+ * @return True if there is more, 0 otherwise.
  */
 static int
 odom_more(struct GETodometer* odom)
@@ -54,8 +62,12 @@ odom_more(struct GETodometer* odom)
     return (odom->index[0] < odom->stop[0]);
 }
 
-/** \internal
-
+/**
+ * @internal Move odometer.
+ *
+ * @param odom Pointer to odometer.
+ *
+ * @return 0 or 1
  */
 static int
 odom_next(struct GETodometer* odom)
@@ -70,7 +82,6 @@ odom_next(struct GETodometer* odom)
     }
     return 1;
 }
-#endif
 
 /** \internal
 \ingroup variables
@@ -82,38 +93,49 @@ NC_get_vara(int ncid, int varid,
             void *value, nc_type memtype)
 {
    NC* ncp;
+   size_t *my_count = (size_t *)edges;
    int stat = NC_check_id(ncid, &ncp);
    if(stat != NC_NOERR) return stat;
-#ifdef USE_NETCDF4
-   if(memtype >= NC_FIRSTUSERTYPEID) memtype = NC_NAT;
-#endif
 
-   if(edges == NULL) {
-      size_t shape[NC_MAX_VAR_DIMS];
-      int ndims;
-      stat = nc_inq_varndims(ncid, varid, &ndims);
+   if(start == NULL || edges == NULL) {
+      stat = NC_check_nulls(ncid, varid, start, &my_count, NULL);
       if(stat != NC_NOERR) return stat;
-      stat = NC_getshape(ncid,varid,ndims,shape);
-      if(stat != NC_NOERR) return stat;
-      stat = ncp->dispatch->get_vara(ncid,varid,start,shape,value,memtype);
-   } else
-      stat =  ncp->dispatch->get_vara(ncid,varid,start,edges,value,memtype);
+   }
+   stat =  ncp->dispatch->get_vara(ncid,varid,start,my_count,value,memtype);
+   if(edges == NULL) free(my_count);
    return stat;
 }
 
-/** \ingroup variables
+/** 
+Get data for a variable.
+
+\param ncid NetCDF or group ID.
+
+\param varid Variable ID
+
+\param value Pointer where the data will be copied. Memory must be
+allocated by the user before this function is called.
+
+\param memtype the NC type of the data after it is read into
+memory. Data are converted from the variable's type to the memtype as
+they are read.
+
+\returns ::NC_NOERR No error.
+\returns ::NC_ENOTVAR Variable not found.
+\returns ::NC_EINVALCOORDS Index exceeds dimension bound.
+\returns ::NC_EEDGE Start+count exceeds dimension bound.
+\returns ::NC_ERANGE One or more of the values are out of range.
+\returns ::NC_EINDEFINE Operation not allowed in define mode.
+\returns ::NC_EBADID Bad ncid.
+
+\ingroup variables
 \internal
+\author Dennis Heimbigner
  */
 static int
 NC_get_var(int ncid, int varid, void *value, nc_type memtype)
 {
-   int ndims;
-   size_t shape[NC_MAX_VAR_DIMS];
-   int stat = nc_inq_varndims(ncid,varid, &ndims);
-   if(stat) return stat;
-   stat = NC_getshape(ncid,varid, ndims, shape);
-   if(stat) return stat;
-   return NC_get_vara(ncid, varid, NC_coord_zero, shape, value, memtype);
+   return NC_get_vara(ncid, varid, NC_coord_zero, NULL, value, memtype);
 }
 
 /** \internal
@@ -125,13 +147,6 @@ NCDEFAULT_get_vars(int ncid, int varid, const size_t * start,
 	    const size_t * edges, const ptrdiff_t * stride,
 	    void *value0, nc_type memtype)
 {
-#ifdef VARS_USES_VARM
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-
-   if(stat != NC_NOERR) return stat;
-   return ncp->dispatch->get_varm(ncid,varid,start,edges,stride,NULL,value0,memtype);
-#else
   /* Rebuilt get_vars code to simplify and avoid use of get_varm */
 
    int status = NC_NOERR;
@@ -142,6 +157,7 @@ NCDEFAULT_get_vars(int ncid, int varid, const size_t * start,
    NC* ncp;
    int memtypelen;
    size_t vartypelen;
+   size_t nels;
    char* value = (char*)value0;
    size_t numrecs;
    size_t varshape[NC_MAX_VAR_DIMS];
@@ -182,6 +198,10 @@ NCDEFAULT_get_vars(int ncid, int varid, const size_t * start,
    status = nc_inq_varndims(ncid, varid, &rank);
    if(status != NC_NOERR) return status;
 
+   /* Start array is always required for non-scalar vars. */
+   if(rank > 0 && start == NULL)
+      return NC_EINVALCOORDS;
+
    /* Get variable dimension sizes */
    isrecvar = NC_is_recvar(ncid,varid,&numrecs);
    NC_getshape(ncid,varid,rank,varshape);
@@ -199,9 +219,18 @@ NCDEFAULT_get_vars(int ncid, int varid, const size_t * start,
 
    /* Do various checks and fixups on start/edges/stride */
    simplestride = 1; /* assume so */
+   nels = 1;
    for(i=0;i<rank;i++) {
 	size_t dimlen;
 	mystart[i] = (start == NULL ? 0 : start[i]);
+        /* illegal value checks */
+	dimlen = (i == 0 && isrecvar ? numrecs : varshape[i]);
+        /* mystart is unsigned, never < 0 */
+#ifdef RELAX_COORD_BOUND
+	if (mystart[i] > dimlen) return NC_EINVALCOORDS;
+#else
+	if (mystart[i] >= dimlen) return NC_EINVALCOORDS;
+#endif
 	if(edges == NULL) {
 	   if(i == 0 && isrecvar)
   	      myedges[i] = numrecs - start[i];
@@ -209,23 +238,23 @@ NCDEFAULT_get_vars(int ncid, int varid, const size_t * start,
 	      myedges[i] = varshape[i] - mystart[i];
 	} else
 	    myedges[i] = edges[i];
-	if(myedges[i] == 0)
-	    return NC_NOERR; /* cannot read anything */
+#ifdef RELAX_COORD_BOUND
+	if (mystart[i] == dimlen && myedges[i] > 0) return NC_EINVALCOORDS;
+#endif
+        /* myedges is unsigned, never < 0 */
+	if(mystart[i] + myedges[i] > dimlen)
+	  return NC_EEDGE;
 	mystride[i] = (stride == NULL ? 1 : stride[i]);
 	if(mystride[i] <= 0
 	   /* cast needed for braindead systems with signed size_t */
            || ((unsigned long) mystride[i] >= X_INT_MAX))
            return NC_ESTRIDE;
   	if(mystride[i] != 1) simplestride = 0;
-        /* illegal value checks */
-	dimlen = (i == 0 && isrecvar ? numrecs : varshape[i]);
-        /* mystart is unsigned, never < 0 */
-	if(mystart[i] >= dimlen)
-	  return NC_EINVALCOORDS;
-        /* myedges is unsigned, never < 0 */
-	if(mystart[i] + myedges[i] > dimlen)
-	  return NC_EEDGE;
+        if(myedges[i] == 0)
+          nels = 0;
    }
+   if(nels == 0)
+      return NC_NOERR; /* cannot read anything */
    if(simplestride) {
       return NC_get_vara(ncid, varid, mystart, myedges, value, memtype);
    }
@@ -251,7 +280,6 @@ NCDEFAULT_get_vars(int ncid, int varid, const size_t * start,
       odom_next(&odom);
    }
    return status;
-#endif
 }
 
 /** \internal
@@ -375,17 +403,26 @@ NCDEFAULT_get_varm(int ncid, int varid, const size_t *start,
       mymap = mystride + varndims;
 
       /*
-       * Initialize I/O parameters.
+       * Check start, edges
        */
       for (idim = maxidim; idim >= 0; --idim)
       {
+	 size_t dimlen =
+	    idim == 0 && isrecvar
+	    ? numrecs
+	    : varshape[idim];
+
 	 mystart[idim] = start != NULL
 	    ? start[idim]
 	    : 0;
 
-	 if (edges != NULL && edges[idim] == 0)
+#ifdef RELAX_COORD_BOUND
+	 if (mystart[idim] > dimlen)
+#else
+	 if (mystart[idim] >= dimlen)
+#endif
 	 {
-	    status = NC_NOERR;    /* read/write no data */
+	    status = NC_EINVALCOORDS;
 	    goto done;
 	 }
 
@@ -404,6 +441,33 @@ NCDEFAULT_get_varm(int ncid, int varid, const size_t *start,
 	    myedges[idim] = varshape[idim] - mystart[idim];
 #endif
 
+#ifdef RELAX_COORD_BOUND
+	 if (mystart[idim] == dimlen && myedges[idim] > 0)
+	 {
+	    status = NC_EINVALCOORDS;
+	    goto done;
+	 }
+#endif
+
+	 if (mystart[idim] + myedges[idim] > dimlen)
+	 {
+	    status = NC_EEDGE;
+	    goto done;
+	 }
+      }
+
+
+      /*
+       * Initialize I/O parameters.
+       */
+      for (idim = maxidim; idim >= 0; --idim)
+      {
+	 if (edges != NULL && edges[idim] == 0)
+	 {
+	    status = NC_NOERR;    /* read/write no data */
+	    goto done;
+	 }
+
 	 mystride[idim] = stride != NULL
 	    ? stride[idim]
 	    : 1;
@@ -429,30 +493,6 @@ NCDEFAULT_get_varm(int ncid, int varid, const size_t *start,
 	 stop[idim] = (mystart[idim] + myedges[idim] * (size_t)mystride[idim]);
       }
 
-      /*
-       * Check start, edges
-       */
-      for (idim = maxidim; idim >= 0; --idim)
-      {
-	 size_t dimlen =
-	    idim == 0 && isrecvar
-	    ? numrecs
-	    : varshape[idim];
-	 if (mystart[idim] >= dimlen)
-	 {
-	    status = NC_EINVALCOORDS;
-	    goto done;
-	 }
-
-	 if (mystart[idim] + myedges[idim] > dimlen)
-	 {
-	    status = NC_EEDGE;
-	    goto done;
-	 }
-
-      }
-
-
       /* Lower body */
       /*
        * As an optimization, adjust I/O parameters when the fastest
@@ -506,27 +546,99 @@ NCDEFAULT_get_varm(int ncid, int varid, const size_t *start,
    return status;
 }
 
-/** \ingroup variables
+/**
+Called by externally visible nc_get_vars_xxx routines.
+
+\param ncid NetCDF or group ID.
+
+\param varid Variable ID
+
+\param start start indices. Required for non-scalar vars.
+
+\param edges count indices.
+
+\param stride data strides.
+
+\param value Pointer where the data will be copied. Memory must be
+allocated by the user before this function is called.
+
+\param memtype the NC type of the data after it is read into
+memory. Data are converted from the variable's type to the memtype as
+they are read.
+
+\returns ::NC_NOERR No error.
+\returns ::NC_ENOTVAR Variable not found.
+\returns ::NC_EINVALCOORDS Index exceeds dimension bound.
+\returns ::NC_EEDGE Start+count exceeds dimension bound.
+\returns ::NC_ERANGE One or more of the values are out of range.
+\returns ::NC_EINDEFINE Operation not allowed in define mode.
+\returns ::NC_EBADID Bad ncid.
+
+\ingroup variables
 \internal
-Called by externally visible nc_get_vars_xxx routines */
+\author Dennis Heimbigner, Ed Hartnett
+*/
 static int
 NC_get_vars(int ncid, int varid, const size_t *start,
 	    const size_t *edges, const ptrdiff_t *stride, void *value,
 	    nc_type memtype)
 {
    NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
+   size_t *my_count = (size_t *)edges;
+   ptrdiff_t *my_stride = (ptrdiff_t *)stride;
+   int stat;
 
+   stat = NC_check_id(ncid, &ncp);
    if(stat != NC_NOERR) return stat;
-#ifdef USE_NETCDF4
-   if(memtype >= NC_FIRSTUSERTYPEID) memtype = NC_NAT;
-#endif
-   return ncp->dispatch->get_vars(ncid,varid,start,edges,stride,value,memtype);
+
+   /* Handle any NULL parameters. */
+   if(start == NULL || edges == NULL || stride == NULL) {
+      stat = NC_check_nulls(ncid, varid, start, &my_count, &my_stride);
+      if(stat != NC_NOERR) return stat;
+   }
+
+   stat = ncp->dispatch->get_vars(ncid,varid,start,my_count,my_stride,
+                                  value,memtype);
+   if(edges == NULL) free(my_count);
+   if(stride == NULL) free(my_stride);
+   return stat;
 }
 
-/** \ingroup variables
+/** 
+Called by externally visible nc_get_varm_xxx routines. Note that the
+varm routines are deprecated. Use the vars routines instead for new
+code.
+
+\param ncid NetCDF or group ID.
+
+\param varid Variable ID
+
+\param start start indices.
+
+\param edges count indices.
+
+\param stride data strides.
+
+\param map mapping of dimensions.
+
+\param value Pointer where the data will be copied. Memory must be
+allocated by the user before this function is called.
+
+\param memtype the NC type of the data after it is read into
+memory. Data are converted from the variable's type to the memtype as
+they are read.
+
+\returns ::NC_NOERR No error.
+\returns ::NC_ENOTVAR Variable not found.
+\returns ::NC_EINVALCOORDS Index exceeds dimension bound.
+\returns ::NC_EEDGE Start+count exceeds dimension bound.
+\returns ::NC_ERANGE One or more of the values are out of range.
+\returns ::NC_EINDEFINE Operation not allowed in define mode.
+\returns ::NC_EBADID Bad ncid.
+
+\ingroup variables
 \internal
-Called by externally visible nc_get_varm_xxx routines
+\author Dennis Heimbigner, Ed Hartnett
  */
 static int
 NC_get_varm(int ncid, int varid, const size_t *start,
@@ -534,13 +646,24 @@ NC_get_varm(int ncid, int varid, const size_t *start,
 	    void *value, nc_type memtype)
 {
    NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
+   size_t *my_count = (size_t *)edges;
+   ptrdiff_t *my_stride = (ptrdiff_t *)stride;
+   int stat;
 
+   stat = NC_check_id(ncid, &ncp);
    if(stat != NC_NOERR) return stat;
-#ifdef USE_NETCDF4
-   if(memtype >= NC_FIRSTUSERTYPEID) memtype = NC_NAT;
-#endif
-   return ncp->dispatch->get_varm(ncid,varid,start,edges,stride,map,value,memtype);
+
+   /* Handle any NULL parameters. */
+   if(start == NULL || edges == NULL || stride == NULL) {
+      stat = NC_check_nulls(ncid, varid, start, &my_count, &my_stride);
+      if(stat != NC_NOERR) return stat;
+   }
+
+   stat = ncp->dispatch->get_varm(ncid, varid, start, my_count, my_stride,
+                                  map, value, memtype);
+   if(edges == NULL) free(my_count);
+   if(stride == NULL) free(my_stride);
+   return stat;
 }
 
 /** \name Reading Data from Variables
@@ -620,13 +743,14 @@ values, five lat values, and ten lon values.
      status = nc_get_vara_double(ncid, rh_id, start, count, rh_vals);
      if (status != NC_NOERR) handle_error(status);
 \endcode
+\author Glenn Davis, Russ Rew, Ed Hartnett, Dennis Heimbigner, Ward Fisher
  */
 /**@{*/
 int
 nc_get_vara(int ncid, int varid, const size_t *startp,
 	    const size_t *countp, void *ip)
 {
-   NC* ncp = NULL;
+   NC* ncp;
    nc_type xtype = NC_NAT;
    int stat = NC_check_id(ncid, &ncp);
    if(stat != NC_NOERR) return stat;
@@ -639,53 +763,34 @@ int
 nc_get_vara_text(int ncid, int varid, const size_t *startp,
 		 const size_t *countp, char *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
-   return NC_get_vara(ncid, varid, startp, countp,
-		      (void *)ip, NC_CHAR);
+   return NC_get_vara(ncid, varid, startp, countp, (void *)ip, NC_CHAR);
 }
 
 int
 nc_get_vara_schar(int ncid, int varid, const size_t *startp,
 		  const size_t *countp, signed char *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
-   return NC_get_vara(ncid, varid, startp, countp,
-		      (void *)ip, NC_BYTE);
+   return NC_get_vara(ncid, varid, startp, countp, (void *)ip, NC_BYTE);
 }
 
 int
 nc_get_vara_uchar(int ncid, int varid, const size_t *startp,
 		  const size_t *countp, unsigned char *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
-   return NC_get_vara(ncid, varid, startp, countp,
-		      (void *)ip, T_uchar);
+   return NC_get_vara(ncid, varid, startp, countp, (void *)ip, T_uchar);
 }
 
 int
 nc_get_vara_short(int ncid, int varid, const size_t *startp,
 		  const size_t *countp, short *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
-   return NC_get_vara(ncid, varid, startp, countp,
-		      (void *)ip, NC_SHORT);
+   return NC_get_vara(ncid, varid, startp, countp, (void *)ip, NC_SHORT);
 }
 
 int
 nc_get_vara_int(int ncid, int varid,
 		const size_t *startp, const size_t *countp, int *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vara(ncid,varid,startp,countp, (void *)ip,NC_INT);
 }
 
@@ -693,9 +798,6 @@ int
 nc_get_vara_long(int ncid, int varid,
 		 const size_t *startp, const size_t *countp, long *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vara(ncid,varid,startp,countp, (void *)ip,T_long);
 }
 
@@ -703,20 +805,13 @@ int
 nc_get_vara_float(int ncid, int varid,
 		  const size_t *startp, const size_t *countp, float *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vara(ncid,varid,startp,countp, (void *)ip,T_float);
 }
 
-
 int
 nc_get_vara_double(int ncid, int varid, const size_t *startp,
 		   const size_t *countp, double *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vara(ncid,varid,startp,countp, (void *)ip,T_double);
 }
 
@@ -724,9 +819,6 @@ int
 nc_get_vara_ubyte(int ncid, int varid,
 		  const size_t *startp, const size_t *countp, unsigned char *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vara(ncid,varid,startp,countp, (void *)ip,T_ubyte);
 }
 
@@ -734,9 +826,6 @@ int
 nc_get_vara_ushort(int ncid, int varid,
 		   const size_t *startp, const size_t *countp, unsigned short *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vara(ncid,varid,startp,countp, (void *)ip,T_ushort);
 }
 
@@ -744,9 +833,6 @@ int
 nc_get_vara_uint(int ncid, int varid,
 		 const size_t *startp, const size_t *countp, unsigned int *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vara(ncid,varid,startp,countp, (void *)ip,T_uint);
 }
 
@@ -754,30 +840,21 @@ int
 nc_get_vara_longlong(int ncid, int varid,
 		     const size_t *startp, const size_t *countp, long long *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vara(ncid,varid,startp,countp, (void *)ip,T_longlong);
 }
 
 int
-nc_get_vara_ulonglong(int ncid, int varid,
-		      const size_t *startp, const size_t *countp, unsigned long long *ip)
+nc_get_vara_ulonglong(int ncid, int varid, const size_t *startp,
+                      const size_t *countp, unsigned long long *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vara(ncid,varid,startp,countp, (void *)ip,NC_UINT64);
 }
 
 #ifdef USE_NETCDF4
 int
-nc_get_vara_string(int ncid, int varid,
-		   const size_t *startp, const size_t *countp, char* *ip)
+nc_get_vara_string(int ncid, int varid, const size_t *startp,
+                   const size_t *countp, char* *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vara(ncid,varid,startp,countp, (void *)ip,NC_STRING);
 }
 
@@ -816,6 +893,7 @@ allocated by the user before this function is called.
 \returns ::NC_ERANGE One or more of the values are out of range.
 \returns ::NC_EINDEFINE Operation not allowed in define mode.
 \returns ::NC_EBADID Bad ncid.
+\author Glenn Davis, Russ Rew, Ed Hartnett, Dennis Heimbigner, Ward Fisher
 */
 /** \{ */
 int
@@ -827,45 +905,30 @@ nc_get_var1(int ncid, int varid, const size_t *indexp, void *ip)
 int
 nc_get_var1_text(int ncid, int varid, const size_t *indexp, char *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var1(ncid, varid, indexp, (void *)ip, NC_CHAR);
 }
 
 int
 nc_get_var1_schar(int ncid, int varid, const size_t *indexp, signed char *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var1(ncid, varid, indexp, (void *)ip, NC_BYTE);
 }
 
 int
 nc_get_var1_uchar(int ncid, int varid, const size_t *indexp, unsigned char *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var1(ncid, varid, indexp, (void *)ip, NC_UBYTE);
 }
 
 int
 nc_get_var1_short(int ncid, int varid, const size_t *indexp, short *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var1(ncid, varid, indexp, (void *)ip, NC_SHORT);
 }
 
 int
 nc_get_var1_int(int ncid, int varid, const size_t *indexp, int *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var1(ncid, varid, indexp, (void *)ip, NC_INT);
 }
 
@@ -873,9 +936,6 @@ int
 nc_get_var1_long(int ncid, int varid, const size_t *indexp,
 		 long *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var1(ncid, varid, indexp, (void *)ip, longtype);
 }
 
@@ -883,9 +943,6 @@ int
 nc_get_var1_float(int ncid, int varid, const size_t *indexp,
 		  float *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var1(ncid, varid, indexp, (void *)ip, NC_FLOAT);
 }
 
@@ -893,9 +950,6 @@ int
 nc_get_var1_double(int ncid, int varid, const size_t *indexp,
 		   double *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var1(ncid, varid, indexp, (void *)ip, NC_DOUBLE);
 }
 
@@ -903,9 +957,6 @@ int
 nc_get_var1_ubyte(int ncid, int varid, const size_t *indexp,
 		  unsigned char *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var1(ncid, varid, indexp, (void *)ip, NC_UBYTE);
 }
 
@@ -913,9 +964,6 @@ int
 nc_get_var1_ushort(int ncid, int varid, const size_t *indexp,
 		   unsigned short *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var1(ncid, varid, indexp, (void *)ip, NC_USHORT);
 }
 
@@ -923,9 +971,6 @@ int
 nc_get_var1_uint(int ncid, int varid, const size_t *indexp,
 		 unsigned int *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var1(ncid, varid, indexp, (void *)ip, NC_UINT);
 }
 
@@ -933,9 +978,6 @@ int
 nc_get_var1_longlong(int ncid, int varid, const size_t *indexp,
 		     long long *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var1(ncid, varid, indexp, (void *)ip, NC_INT64);
 }
 
@@ -943,9 +985,6 @@ int
 nc_get_var1_ulonglong(int ncid, int varid, const size_t *indexp,
 		      unsigned long long *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var1(ncid, varid, indexp, (void *)ip, NC_UINT64);
 }
 
@@ -953,9 +992,6 @@ nc_get_var1_ulonglong(int ncid, int varid, const size_t *indexp,
 int
 nc_get_var1_string(int ncid, int varid, const size_t *indexp, char* *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var1(ncid, varid, indexp, (void *)ip, NC_STRING);
 }
 #endif /*USE_NETCDF4*/
@@ -1002,6 +1038,7 @@ allocated by the user before this function is called.
 \returns ::NC_ERANGE One or more of the values are out of range.
 \returns ::NC_EINDEFINE Operation not allowed in define mode.
 \returns ::NC_EBADID Bad ncid.
+\author Glenn Davis, Russ Rew, Ed Hartnett, Dennis Heimbigner, Ward Fisher
 */
 /** \{ */
 int
@@ -1013,117 +1050,78 @@ nc_get_var(int ncid, int varid, void *ip)
 int
 nc_get_var_text(int ncid, int varid, char *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var(ncid, varid, (void *)ip, NC_CHAR);
 }
 
 int
 nc_get_var_schar(int ncid, int varid, signed char *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var(ncid, varid, (void *)ip, NC_BYTE);
 }
 
 int
 nc_get_var_uchar(int ncid, int varid, unsigned char *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var(ncid,varid, (void *)ip, NC_UBYTE);
 }
 
 int
 nc_get_var_short(int ncid, int varid, short *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var(ncid, varid, (void *)ip, NC_SHORT);
 }
 
 int
 nc_get_var_int(int ncid, int varid, int *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var(ncid,varid, (void *)ip, NC_INT);
 }
 
 int
 nc_get_var_long(int ncid, int varid, long *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var(ncid,varid, (void *)ip, longtype);
 }
 
 int
 nc_get_var_float(int ncid, int varid, float *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var(ncid,varid, (void *)ip, NC_FLOAT);
 }
 
 int
 nc_get_var_double(int ncid, int varid, double *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var(ncid,varid, (void *)ip, NC_DOUBLE);
 }
 
 int
 nc_get_var_ubyte(int ncid, int varid, unsigned char *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var(ncid,varid, (void *)ip, NC_UBYTE);
 }
 
 int
 nc_get_var_ushort(int ncid, int varid, unsigned short *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var(ncid,varid, (void *)ip, NC_USHORT);
 }
 
 int
 nc_get_var_uint(int ncid, int varid, unsigned int *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var(ncid,varid, (void *)ip, NC_UINT);
 }
 
 int
 nc_get_var_longlong(int ncid, int varid, long long *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var(ncid,varid, (void *)ip, NC_INT64);
 }
 
 int
 nc_get_var_ulonglong(int ncid, int varid, unsigned long long *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var(ncid,varid, (void *)ip,NC_UINT64);
 }
 
@@ -1131,9 +1129,6 @@ nc_get_var_ulonglong(int ncid, int varid, unsigned long long *ip)
 int
 nc_get_var_string(int ncid, int varid, char* *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_var(ncid,varid, (void *)ip,NC_STRING);
 }
 #endif /*USE_NETCDF4*/
@@ -1177,19 +1172,15 @@ allocated by the user before this function is called.
 \returns ::NC_ERANGE One or more of the values are out of range.
 \returns ::NC_EINDEFINE Operation not allowed in define mode.
 \returns ::NC_EBADID Bad ncid.
+\author Glenn Davis, Russ Rew, Ed Hartnett, Dennis Heimbigner, Ward Fisher
 */
 /** \{ */
 int
-nc_get_vars (int ncid, int varid, const size_t * startp,
-	     const size_t * countp, const ptrdiff_t * stridep,
-	     void *ip)
+nc_get_vars(int ncid, int varid, const size_t * startp,
+            const size_t * countp, const ptrdiff_t * stridep,
+            void *ip)
 {
-   NC* ncp;
-   int stat = NC_NOERR;
-
-   if ((stat = NC_check_id(ncid, &ncp)))
-       return stat;
-   return ncp->dispatch->get_vars(ncid, varid, startp, countp, stridep,
+   return NC_get_vars(ncid, varid, startp, countp, stridep,
 		      ip, NC_NAT);
 }
 
@@ -1198,9 +1189,6 @@ nc_get_vars_text(int ncid, int varid, const size_t *startp,
 		 const size_t *countp, const ptrdiff_t * stridep,
 		 char *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vars(ncid,varid,startp, countp, stridep,
 		      (void *)ip, NC_CHAR);
 }
@@ -1210,9 +1198,6 @@ nc_get_vars_schar(int ncid, int varid, const size_t *startp,
 		  const size_t *countp, const ptrdiff_t * stridep,
 		  signed char *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vars(ncid,varid,startp, countp, stridep,
 		      (void *)ip, NC_BYTE);
 }
@@ -1222,9 +1207,6 @@ nc_get_vars_uchar(int ncid, int varid, const size_t *startp,
 		  const size_t *countp, const ptrdiff_t * stridep,
 		  unsigned char *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vars(ncid,varid,startp, countp, stridep,
 		      (void *)ip, T_uchar);
 }
@@ -1234,9 +1216,6 @@ nc_get_vars_short(int ncid, int varid, const size_t *startp,
 		  const size_t *countp, const ptrdiff_t *stridep,
 		  short *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vars(ncid,varid,startp, countp, stridep,
 		      (void *)ip, NC_SHORT);
 }
@@ -1246,9 +1225,6 @@ nc_get_vars_int(int ncid, int varid, const size_t *startp,
 		const size_t *countp, const ptrdiff_t * stridep,
 		int *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vars(ncid,varid,startp, countp, stridep,
 		      (void *)ip, NC_INT);
 }
@@ -1258,9 +1234,6 @@ nc_get_vars_long(int ncid, int varid, const size_t *startp,
 		 const size_t *countp, const ptrdiff_t * stridep,
 		 long *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vars(ncid,varid,startp, countp, stridep,
 		      (void *)ip, T_long);
 }
@@ -1270,9 +1243,6 @@ nc_get_vars_float(int ncid, int varid, const size_t *startp,
 		  const size_t *countp, const ptrdiff_t * stridep,
 		  float *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vars(ncid,varid,startp, countp, stridep,
 		      (void *)ip, T_float);
 }
@@ -1282,9 +1252,6 @@ nc_get_vars_double(int ncid, int varid, const size_t *startp,
 		   const size_t *countp, const ptrdiff_t * stridep,
 		   double *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vars(ncid,varid,startp, countp, stridep,
 		      (void *)ip, T_double);
 }
@@ -1294,9 +1261,6 @@ nc_get_vars_ubyte(int ncid, int varid, const size_t *startp,
 		  const size_t *countp, const ptrdiff_t * stridep,
 		  unsigned char *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vars(ncid,varid, startp, countp, stridep,
 		      (void *)ip, T_ubyte);
 }
@@ -1306,9 +1270,6 @@ nc_get_vars_ushort(int ncid, int varid, const size_t *startp,
 		   const size_t *countp, const ptrdiff_t * stridep,
 		   unsigned short *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vars(ncid,varid,startp,countp, stridep,
 		      (void *)ip, T_ushort);
 }
@@ -1318,9 +1279,6 @@ nc_get_vars_uint(int ncid, int varid, const size_t *startp,
 		 const size_t *countp, const ptrdiff_t * stridep,
 		 unsigned int *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vars(ncid,varid,startp, countp, stridep,
 		      (void *)ip, T_uint);
 }
@@ -1330,9 +1288,6 @@ nc_get_vars_longlong(int ncid, int varid, const size_t *startp,
 		     const size_t *countp, const ptrdiff_t * stridep,
 		     long long *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vars(ncid, varid, startp, countp, stridep,
 		      (void *)ip, T_longlong);
 }
@@ -1342,9 +1297,6 @@ nc_get_vars_ulonglong(int ncid, int varid, const size_t *startp,
 		      const size_t *countp, const ptrdiff_t * stridep,
 		      unsigned long long *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vars(ncid, varid, startp, countp, stridep,
 		      (void *)ip, NC_UINT64);
 }
@@ -1356,9 +1308,6 @@ nc_get_vars_string(int ncid, int varid,
 		   const ptrdiff_t * stridep,
 		   char* *ip)
 {
-   NC* ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_vars(ncid, varid, startp, countp, stridep,
 		      (void *)ip, NC_STRING);
 }
@@ -1418,6 +1367,7 @@ allocated by the user before this function is called.
 \returns ::NC_ERANGE One or more of the values are out of range.
 \returns ::NC_EINDEFINE Operation not allowed in define mode.
 \returns ::NC_EBADID Bad ncid.
+\author Glenn Davis, Russ Rew, Ed Hartnett, Dennis Heimbigner, Ward Fisher
 */
 /** \{ */
 int
@@ -1425,13 +1375,7 @@ nc_get_varm(int ncid, int varid, const size_t * startp,
 	    const size_t * countp, const ptrdiff_t * stridep,
 	    const ptrdiff_t * imapp, void *ip)
 {
-   NC* ncp;
-   int stat = NC_NOERR;
-
-   if ((stat = NC_check_id(ncid, &ncp)))
-       return stat;
-   return ncp->dispatch->get_varm(ncid, varid, startp, countp,
-				  stridep, imapp, ip, NC_NAT);
+   return NC_get_varm(ncid, varid, startp, countp, stridep, imapp, ip, NC_NAT);
 }
 
 int
@@ -1440,9 +1384,6 @@ nc_get_varm_schar(int ncid, int varid,
 		  const ptrdiff_t *stridep,
 		  const ptrdiff_t *imapp, signed char *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_varm(ncid, varid, startp, countp,
 		      stridep, imapp, (void *)ip, NC_BYTE);
 }
@@ -1453,9 +1394,6 @@ nc_get_varm_uchar(int ncid, int varid,
 		  const ptrdiff_t *stridep, const ptrdiff_t *imapp,
 		  unsigned char *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_varm(ncid,varid,startp,countp,stridep,imapp, (void *)ip,T_uchar);
 }
 
@@ -1464,9 +1402,6 @@ nc_get_varm_short(int ncid, int varid, const size_t *startp,
 		  const size_t *countp, const ptrdiff_t *stridep,
 		  const ptrdiff_t *imapp, short *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_varm(ncid,varid,startp,countp,stridep,imapp, (void *)ip,NC_SHORT);
 }
 
@@ -1476,9 +1411,6 @@ nc_get_varm_int(int ncid, int varid,
 		const ptrdiff_t *stridep, const ptrdiff_t *imapp,
 		int *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_varm(ncid,varid,startp,countp,stridep,imapp, (void *)ip,NC_INT);
 }
 
@@ -1488,9 +1420,6 @@ nc_get_varm_long(int ncid, int varid,
 		 const ptrdiff_t *stridep, const ptrdiff_t *imapp,
 		 long *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_varm(ncid,varid,startp,countp,stridep,imapp, (void *)ip,T_long);
 }
 
@@ -1500,9 +1429,6 @@ nc_get_varm_float(int ncid, int varid,
 		  const ptrdiff_t *stridep, const ptrdiff_t *imapp,
 		  float *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_varm(ncid,varid,startp,countp,stridep,imapp, (void *)ip,T_float);
 }
 
@@ -1512,9 +1438,6 @@ nc_get_varm_double(int ncid, int varid,
 		   const ptrdiff_t *stridep, const ptrdiff_t *imapp,
 		   double *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_varm(ncid,varid,startp,countp,stridep,imapp, (void *)ip,T_double);
 }
 
@@ -1524,9 +1447,6 @@ nc_get_varm_ubyte(int ncid, int varid,
 		  const ptrdiff_t *stridep, const ptrdiff_t *imapp,
 		  unsigned char *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_varm(ncid,varid,startp,countp,stridep,
 		      imapp, (void *)ip, T_ubyte);
 }
@@ -1537,9 +1457,6 @@ nc_get_varm_ushort(int ncid, int varid,
 		   const ptrdiff_t *stridep, const ptrdiff_t *imapp,
 		   unsigned short *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_varm(ncid, varid, startp, countp, stridep,
 		      imapp, (void *)ip, T_ushort);
 }
@@ -1550,9 +1467,6 @@ nc_get_varm_uint(int ncid, int varid,
 		 const ptrdiff_t *stridep, const ptrdiff_t *imapp,
 		 unsigned int *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_varm(ncid, varid, startp, countp,
 		      stridep, imapp, (void *)ip, T_uint);
 }
@@ -1562,9 +1476,6 @@ nc_get_varm_longlong(int ncid, int varid, const size_t *startp,
 		     const size_t *countp, const ptrdiff_t *stridep,
 		     const ptrdiff_t *imapp, long long *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_varm(ncid, varid, startp, countp, stridep, imapp,
 		      (void *)ip, T_longlong);
 }
@@ -1575,9 +1486,6 @@ nc_get_varm_ulonglong(int ncid, int varid,
 		      const ptrdiff_t *stridep, const ptrdiff_t *imapp,
 		      unsigned long long *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_varm(ncid, varid, startp, countp, stridep, imapp,
 		      (void *)ip, NC_UINT64);
 }
@@ -1587,9 +1495,6 @@ nc_get_varm_text(int ncid, int varid, const size_t *startp,
 		 const size_t *countp, const ptrdiff_t *stridep,
 		 const ptrdiff_t *imapp, char *ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_varm(ncid, varid, startp, countp, stridep, imapp,
 		      (void *)ip, NC_CHAR);
 }
@@ -1600,9 +1505,6 @@ nc_get_varm_string(int ncid, int varid, const size_t *startp,
 		   const size_t *countp, const ptrdiff_t *stridep,
 		   const ptrdiff_t *imapp, char **ip)
 {
-   NC *ncp;
-   int stat = NC_check_id(ncid, &ncp);
-   if(stat != NC_NOERR) return stat;
    return NC_get_varm(ncid, varid, startp, countp, stridep, imapp,
 		      (void *)ip, NC_STRING);
 }
