@@ -5,7 +5,11 @@
 /* $Id: winceio.c,v 1.2 2010/05/04 17:30:04 dmh Exp $ */
 /* Dennis Heimbigner 2010-3-04 */
 
+
+#if HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <assert.h>
 #include <stdlib.h>
 #include <stdio.h>	/* DEBUG */
@@ -20,8 +24,8 @@
 #define EEXIST NC_EEXIST
 #endif
 
-#ifndef ENOERR
-#define ENOERR 0
+#ifndef NC_NOERR
+#define NC_NOERR 0
 #endif
 
 #include <string.h>
@@ -65,16 +69,16 @@ blksize(int fd)
 static int
 fgrow(FILE* f, const off_t len)
 {
-    int status = ENOERR;
+    int status = NC_NOERR;
     long pos = ftell(f);
     long size;
-    pos = ftell(f);    
+    pos = ftell(f);
     status = fseek(f,0,SEEK_END);
     if(ferror(f)) return EIO;
     size = ftell(f);
     status = fseek(f,pos,SEEK_SET);
     if(ferror(f)) return EIO;
-    if(len < size) return ENOERR;
+    if(len < size) return NC_NOERR;
     else {
 	const long dumb = 0;
 	status = fseek(f, len-sizeof(dumb), SEEK_SET);
@@ -84,7 +88,7 @@ fgrow(FILE* f, const off_t len)
 	status = fseek(f, pos, SEEK_SET);
         if(ferror(f)) return EIO;
     }
-    return ENOERR;
+    return NC_NOERR;
 }
 
 
@@ -96,16 +100,16 @@ fgrow(FILE* f, const off_t len)
 static int
 fgrow2(FILE* f, const off_t len)
 {
-    int status = ENOERR;
+    int status = NC_NOERR;
     long pos = ftell(f);
     long size;
-    pos = ftell(f);    
+    pos = ftell(f);
     status = fseek(f,0,SEEK_END);
     if(ferror(f)) return EIO;
     size = ftell(f);
     status = fseek(f,pos,SEEK_SET);
     if(ferror(f)) return EIO;
-    if(len < size) return ENOERR;
+    if(len < size) return NC_NOERR;
     else {
 	const char dumb = 0;
 	status = fseek(f, len-sizeof(dumb), SEEK_SET);
@@ -115,17 +119,17 @@ fgrow2(FILE* f, const off_t len)
 	status = fseek(f, pos, SEEK_SET);
 	if(ferror(f)) return EIO;
     }
-    return ENOERR;
+    return NC_NOERR;
 }
 /* End OS */
 /* Begin ffio */
 
 static int
-fileio_pgout(ncio *const nciop, 
+fileio_pgout(ncio *const nciop,
 	off_t const offset,  const size_t extent,
 	const void *const vp, off_t *posp)
 {
-       int status = ENOERR;
+       int status = NC_NOERR;
        FILE* f = descriptors[nciop->fd];
 
 #ifdef X_ALIGN
@@ -142,7 +146,7 @@ fileio_pgout(ncio *const nciop,
 	fwrite(vp,1,extent,f);
         if(ferror(f)) return EIO;
 	*posp += extent;
-	return ENOERR;
+	return NC_NOERR;
 }
 
 static int
@@ -150,7 +154,7 @@ fileio_pgin(ncio *const nciop,
 	off_t const offset, const size_t extent,
 	void *const vp, size_t *nreadp, off_t *posp)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
 	ssize_t nread;
 	int count;
 
@@ -172,7 +176,7 @@ fileio_pgin(ncio *const nciop,
         if(ferror(f)) return EIO;
 	*nreadp = nread;
 	*posp += nread;
-	return ENOERR;
+	return NC_NOERR;
 }
 
 /* */
@@ -180,7 +184,7 @@ fileio_pgin(ncio *const nciop,
 typedef struct ncio_ffio {
 	off_t pos;
 	/* buffer */
-	off_t	bf_offset; 
+	off_t	bf_offset;
 	size_t	bf_extent;
 	size_t	bf_cnt;
 	void	*bf_base;
@@ -189,7 +193,7 @@ typedef struct ncio_ffio {
 static int
 ncio_fileio_rel(ncio *const nciop, off_t offset, int rflags)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
         FILE* f = descriptors[nciop->fd];
 
 	ncio_ffio *ffp = (ncio_ffio *)nciop->pvt;
@@ -225,12 +229,12 @@ ncio_fileio_get(ncio *const nciop,
 		void **const vpp)
 {
 	ncio_ffio *ffp = (ncio_ffio *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
         FILE* f = descriptors[nciop->fd];
 #ifdef X_ALIGN
 	size_t rem;
 #endif
-	
+
 	if(fIsSet(rflags, RGN_WRITE) && !fIsSet(nciop->ioflags, NC_WRITE))
 		return EPERM; /* attempt to write readonly file */
 
@@ -277,7 +281,7 @@ ncio_fileio_get(ncio *const nciop,
 		 extent,
 		 ffp->bf_base,
 		 &ffp->bf_cnt, &ffp->pos);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	ffp->bf_offset = offset;
@@ -295,7 +299,7 @@ ncio_fileio_get(ncio *const nciop,
 #else
 	*vpp = (char *)ffp->bf_base;
 #endif
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -303,8 +307,8 @@ static int
 ncio_fileio_move(ncio *const nciop, off_t to, off_t from,
 			size_t nbytes, int rflags)
 {
-	int status = ENOERR;
-	off_t lower = from;	
+	int status = NC_NOERR;
+	off_t lower = from;
 	off_t upper = to;
 	char *base;
 	size_t diff = upper - lower;
@@ -314,12 +318,12 @@ ncio_fileio_move(ncio *const nciop, off_t to, off_t from,
 	rflags &= RGN_NOLOCK; /* filter unwanted flags */
 
 	if(to == from)
-		return ENOERR; /* NOOP */
-	
+		return NC_NOERR; /* NOOP */
+
 	if(to > from)
 	{
 		/* growing */
-		lower = from;	
+		lower = from;
 		upper = to;
 	}
 	else
@@ -335,14 +339,14 @@ ncio_fileio_move(ncio *const nciop, off_t to, off_t from,
 	status = ncio_fileio_get(nciop, lower, extent, RGN_WRITE|rflags,
 			(void **)&base);
 
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	if(to > from)
-		(void) memmove(base + diff, base, nbytes); 
+		(void) memmove(base + diff, base, nbytes);
 	else
-		(void) memmove(base, base + diff, nbytes); 
-		
+		(void) memmove(base, base + diff, nbytes);
+
 	(void) ncio_fileio_rel(nciop, lower, RGN_MODIFIED);
 
 	return status;
@@ -353,7 +357,7 @@ ncio_fileio_sync(ncio *const nciop)
 {
         FILE* f = descriptors[nciop->fd];
 	fflush(f);
-	return ENOERR;
+	return NC_NOERR;
 }
 
 static void
@@ -394,7 +398,7 @@ ncio_fileio_init2(ncio *const nciop, size_t *sizehintp)
 		return ENOMEM;
 	}
 	/* else */
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -426,7 +430,7 @@ ncio_free(ncio *nciop)
 
 	if(nciop->free != NULL)
 		nciop->free(nciop->pvt);
-	
+
 	free(nciop);
 }
 
@@ -437,7 +441,7 @@ ncio_new(const char *path, int ioflags)
 	size_t sz_path = M_RNDUP(strlen(path) +1);
 	size_t sz_ncio_pvt;
 	ncio *nciop;
- 
+
 #if ALWAYS_NC_SHARE /* DEBUG */
 	fSet(ioflags, NC_SHARE);
 #endif
@@ -450,7 +454,7 @@ ncio_new(const char *path, int ioflags)
 	nciop = (ncio *) malloc(sz_ncio + sz_path + sz_ncio_pvt);
 	if(nciop == NULL)
 		return NULL;
-	
+
 	nciop->ioflags = ioflags;
 	*((int *)&nciop->fd) = -1; /* cast away const */
 
@@ -486,7 +490,7 @@ ncio_create(const char *path, int ioflags,
 #endif
 	FILE* f;
 	int i,fd;
-	int status = ENOERR;
+	int status = NC_NOERR;
 
 	if(initialsz < (size_t)igeto + igetsz)
 		initialsz = (size_t)igeto + igetsz;
@@ -511,7 +515,7 @@ ncio_create(const char *path, int ioflags,
 	    if(f != NULL) { /* do not overwrite */
 		(void)fclose(f);
 		return EEXIST;
-	    }		
+	    }
 	}
 
 	f = fopen(path, oflags);
@@ -539,13 +543,13 @@ ncio_create(const char *path, int ioflags,
 	}
 
 	status = ncio_fileio_init2(nciop, sizehintp);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		goto unwind_open;
 
 	if(initialsz != 0)
 	{
 		status = fgrow(f, (off_t)initialsz);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			goto unwind_open;
 	}
 
@@ -555,12 +559,12 @@ ncio_create(const char *path, int ioflags,
 				igeto, igetsz,
                         	RGN_WRITE,
                         	igetvpp);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			goto unwind_open;
 	}
 
 	*nciopp = nciop;
-	return ENOERR;
+	return NC_NOERR;
 
 unwind_open:
 	(void) fclose(descriptors[fd]);
@@ -589,7 +593,7 @@ ncio_open(const char *path,
 #endif
 	FILE* f;
 	int i,fd;
-	int status = ENOERR;
+	int status = NC_NOERR;
 
 	if(path == NULL || *path == 0)
 		return EINVAL;
@@ -625,7 +629,7 @@ ncio_open(const char *path,
 	}
 
 	status = ncio_fileio_init2(nciop, sizehintp);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		goto unwind_open;
 
 	if(igetsz != 0)
@@ -634,12 +638,12 @@ ncio_open(const char *path,
 				igeto, igetsz,
                         	0,
                         	igetvpp);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			goto unwind_open;
 	}
 
 	*nciopp = nciop;
-	return ENOERR;
+	return NC_NOERR;
 
 unwind_open:
 	(void) fclose(descriptors[fd]);
@@ -651,15 +655,15 @@ unwind_new:
 }
 
 
-/* 
- * Get file size in bytes.  
+/*
+ * Get file size in bytes.
  * Is use of fstatus = fseek() really necessary, or could we use standard fstat() call
  * and get st_size member?
  */
 int
 ncio_filesize(ncio *nciop, off_t *filesizep)
 {
-    int status = ENOERR;
+    int status = NC_NOERR;
     off_t filesize, current, reset;
     FILE* f;
 
@@ -672,9 +676,9 @@ ncio_filesize(ncio *nciop, off_t *filesizep)
     status = fseek(f, 0, SEEK_END); /* get size */
     if(ferror(f)) return EIO;
     *filesizep = ftell(f);
-    status = fseek(f, current, SEEK_SET); /* reset */ 
+    status = fseek(f, current, SEEK_SET); /* reset */
     if(ferror(f)) return EIO;
-    return ENOERR;
+    return NC_NOERR;
 }
 
 
@@ -688,7 +692,7 @@ ncio_filesize(ncio *nciop, off_t *filesizep)
 int
 ncio_pad_length(ncio *nciop, off_t length)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
 	FILE* f;
 
 	if(nciop == NULL)
@@ -700,20 +704,20 @@ ncio_pad_length(ncio *nciop, off_t length)
 	        return EPERM; /* attempt to write readonly file */
 
 	status = nciop->sync(nciop);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 	        return status;
 
 	status = fgrow2(f, length);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 	        return errno;
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
-int 
+int
 ncio_close(ncio *nciop, int doUnlink)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
 	FILE* f;
 
 	if(nciop == NULL)
@@ -726,7 +730,7 @@ ncio_close(ncio *nciop, int doUnlink)
 	(void) fclose(f);
 
 	descriptors[nciop->fd] = NULL;
-	
+
 	if(doUnlink)
 		(void) unlink(nciop->path);
 
