@@ -38,10 +38,34 @@
 static int status = NC_NOERR;
 
 /* Control flags  */
-static int persist, usenetcdf4, mmap, diskless;
+static int persist, usenetcdf4, mmap, diskless, file;
 
 static int diskmode;
 
+char*
+smode(int mode)
+{
+    static char ms[8192];
+    ms[0] = '\0';
+    if(mode & NC_NETCDF4)
+	strcat(ms,"NC_NETCDF4");
+    else
+	strcat(ms,"NC_NETCDF3");
+    if(mode & NC_DISKLESS)
+	strcat(ms,"|NC_DISKLESS");
+    if(mode & NC_WRITE)
+	strcat(ms,"|NC_WRITE");
+    if(mode & NC_NOCLOBBER)
+	strcat(ms,"|NC_NOCLOBBER");
+    if(mode & NC_INMEMORY)
+	strcat(ms,"|NC_INMEMORY");
+    if(mode & NC_PERSIST)
+	strcat(ms,"|NC_PERSIST");
+    if(mode & NC_MMAP)
+	strcat(ms,"|NC_MMAP");
+    return ms;
+}
+
 /* Test a diskless file with two record vars, which grow, and has
  * attributes added. */
 static int
@@ -54,23 +78,18 @@ test_two_growing_with_att(const char *testfile)
    int v, r;
 
    /* Create a file with one ulimited dimensions, and one var. */
-   if((status=nc_create(testfile, NC_CLOBBER, &ncid))) ERRSTAT(status);
+   if((status=nc_create(testfile, diskmode|NC_CLOBBER, &ncid))) ERRSTAT(status);
    if((status=nc_def_dim(ncid, DIM1_NAME, NC_UNLIMITED, &dimid))) ERRSTAT(status);
    if((status=nc_def_var(ncid, VAR_NAME, NC_CHAR, 1, &dimid, &varid[0]))) ERRSTAT(status);
    if((status=nc_def_var(ncid, VAR_NAME2, NC_CHAR, 1, &dimid, &varid[1]))) ERRSTAT(status);
-   if((status=nc_close(ncid))) ERRSTAT(status);
+   if((status=nc_enddef(ncid))) ERRSTAT(status);
 
    /* Create some phoney data. */
    for (data[0] = 'a', r = 1; r < MAX_RECS; r++)
       data[r] = data[r - 1] + 1;
 
-   /* Normally one would not close and reopen the file for each
-    * record, nor add an attribute each time I add a record, but I am
-    * giving the library a little work-out here... */
    for (r = 0; r < MAX_RECS; r++)
    {
-      /* Write one record of var data, a single character. */
-      if((status=nc_open(testfile, NC_WRITE, &ncid))) ERRSTAT(status);
       count[0] = 1;
       start[0] = r;
       sprintf(att_name, "a_%d", data[r]);
@@ -81,10 +100,8 @@ test_two_growing_with_att(const char *testfile)
 	 if((status=nc_put_att_text(ncid, varid[v], att_name, 1, &data[r]))) ERRSTAT(status);
 	 if((status=nc_enddef(ncid))) ERRSTAT(status);
       }
-      if((status=nc_close(ncid))) ERRSTAT(status);
 
-      /* Reopen the file and check it. */
-      if((status=nc_open(testfile, diskmode|NC_WRITE, &ncid))) ERRSTAT(status);
+      /* verify */
       if((status=nc_inq_dimlen(ncid, 0, &len_in))) ERRSTAT(status);
       if (len_in != r + 1) ERR;
       index[0] = r;
@@ -93,8 +110,8 @@ test_two_growing_with_att(const char *testfile)
 	 if((status=nc_get_var1_text(ncid, varid[v], index, &data_in))) ERRSTAT(status);
 	 if (data_in != data[r]) ERR;
       }
-      if((status=nc_close(ncid))) ERRSTAT(status);
    } /* Next record. */
+  if((status=nc_close(ncid))) ERRSTAT(status);
    return 0;
 }
 
@@ -146,24 +163,36 @@ main(int argc, char **argv)
     usenetcdf4 = 0;
     mmap = 0;
     diskless = 0;
+    file = 0;
     diskmode = 0;
 
     for(i=1;i<argc;i++) {
 	if(strcmp(argv[i],"diskless")==0) diskless=1;
 	else if(strcmp(argv[i],"mmap")==0) mmap=1;
+	else if(strcmp(argv[i],"file")==0) file=1;
+	else if(strcmp(argv[i],"persist")==0) persist=1;
 	/* ignore anything not recognized */
     }
 
+    if(diskless && mmap) {
+	fprintf(stderr,"NC_DISKLESS and NC_MMAP are mutually exclusive\n");
+	exit(1);
+    }
+
+    if(!diskless && !mmap && !file) {
+	fprintf(stderr,"file or diskless or mmap must be specified\n");
+	exit(1);
+    }
+
     if(diskless)
         diskmode |= NC_DISKLESS;
-    if(diskless && mmap)
+    if(mmap)
         diskmode |= NC_MMAP;
+    if(persist)
+        diskmode |= NC_PERSIST;
 
-    printf("\n*** Testing diskless file: create/modify %s",
-	    diskless?"in-memory":"in-file");
-    if(diskless && mmap)
-        printf("+mmap");
-    printf(" %s\n",NCFILENAME);
+    printf("\n*** Testing create/modify file=%s mode=%s\n", NCFILENAME,
+	    diskless?"diskless":"mmap");
 
     /* case NC_FORMAT_CLASSIC: only test this format */
     nc_set_default_format(NC_FORMAT_CLASSIC, NULL);
