@@ -1,3 +1,4 @@
+#include "config.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -81,9 +82,9 @@ main()
 
     topsrcdir = gettopsrcdir();
 
-    strcpy(url,"file://");
-    strcat(url,topsrcdir);
-    strcat(url,"/ncdap_test/testdata3/test.06");
+    strncpy(url,"file://",sizeof(url));
+    strlcat(url,topsrcdir,sizeof(url));
+    strlcat(url,"/ncdap_test/testdata3/test.06",sizeof(url));
 
     printf("test_vara: url=%s\n",url);
 
